import os
import tempfile


class ObpAuthProvider(object):
    name = "obp"

    @staticmethod
    def get_client(
        module, with_error=False, role_arn=None, session_vars=None, client_params=None
    ):
        if client_params is None:
            client_params = {}

        import boto3
        import botocore
        from botocore.exceptions import ClientError
        from metaflow_extensions.outerbounds.plugins.auth_server import get_token

        token_info = get_token("/generate/aws")

        token_file = "/tmp/obp_token"

        # Write to a temp file then rename to avoid a situation when someone
        # tries to read the file after it was open for writing (and truncated)
        # but before the token was written to it.
        with tempfile.NamedTemporaryFile("w", delete=False) as f:
            f.write(token_info["token"])
            tmp_token_file = f.name
        os.rename(tmp_token_file, token_file)

        os.environ["AWS_WEB_IDENTITY_TOKEN_FILE"] = token_file
        os.environ["AWS_ROLE_ARN"] = token_info["role_arn"]

        if role_arn:
            session = boto3.session.Session()
            fetcher = botocore.credentials.AssumeRoleCredentialFetcher(
                client_creator=session._session.create_client,
                source_credentials=session._session.get_credentials(),
                role_arn=role_arn,
                extra_args={},
            )
            creds = botocore.credentials.DeferredRefreshableCredentials(
                method="assume-role", refresh_using=fetcher.fetch_credentials
            )
            botocore_session = botocore.session.Session(session_vars=session_vars)
            botocore_session._credentials = creds
            session = boto3.session.Session(botocore_session=botocore_session)
            if with_error:
                return session.client(module, **client_params), ClientError
            else:
                return session.client(module, **client_params)
        if with_error:
            return boto3.client(module, **client_params), ClientError
        else:
            return boto3.client(module, **client_params)


AWS_CLIENT_PROVIDERS = [ObpAuthProvider]



import os
import requests
from urllib.parse import urlparse


def get_token(url_path):
    from metaflow.metaflow_config import (
        METADATA_SERVICE_HEADERS,
        from_conf,
        METADATA_SERVICE_URL,
    )
    from metaflow.exception import MetaflowException

    # Infer auth host from metadata service URL, unless it has been
    # specified explicitly. Take the MDS host and replace first part of
    # the domain name with `auth.`. All our deployments follow this scheme
    # anyways.
    #
    auth_host = "auth." + urlparse(METADATA_SERVICE_URL).hostname.split(".", 1)[1]

    authServer = from_conf("OBP_AUTH_SERVER", auth_host)
    assert url_path.startswith("/")
    url = "https://" + authServer + url_path
    headers = METADATA_SERVICE_HEADERS
    try:
        r = requests.get(url, headers=headers)
        r.raise_for_status()
        token_info = r.json()
        return token_info

    except requests.exceptions.HTTPError as e:
        raise MetaflowException(repr(e))
